using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Utility.ModifyRegistry;

/*
 * Apple Macbook Pro SMC control
 *
 * Copyright (C) 2006 Nicolas Boichat <nicolas@boichat.ch>
 * Copyright (C) 2006 Niklas Jakob (Windows port)
 * Copyright (C) 2006 Rick Adair (.Net Tray Application Port)
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
namespace KeyboardDimmer
{
    public partial class MainForm : Form
    {
        private bool _appExit = false;

        #region Form Methods
        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            ModifyRegistry _registry = new ModifyRegistry();

            init_motion_sensor();

            try
            {
                TBThreshold.Text = _registry.Read("Off_Threshold");
            }
            catch {}

            if (_registry.Read("Use_Internal_Sensor") == "true")
                chkUseInternalSensor.Checked = true;
            else
            {
                chkUseInternalSensor.Checked = false;
                try
                {
                    TBBrightness.Value = Convert.ToInt16(_registry.Read("Manual_Sensor_Setting"));
                }
                catch {}
            }

            BW.RunWorkerAsync();
        }

        private void MainForm_Deactivate(object sender, EventArgs e)
        {
            this.Opacity = 0;
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            _appExit = true;
            InternalSetBacklight(0);
        }
        #endregion

        #region Form Control Methods
        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _appExit = true;
            I.Dispose();
            ssleep(1000);
            BW.CancelAsync();
            BW.Dispose();
            Application.Exit();
            
            InternalSetBacklight(0);
        }

        private void I_Click(object sender, EventArgs e)
        {
            if (((MouseEventArgs)e).Button == MouseButtons.Left)
            {
                this.Opacity = 100;
                this.Activate();

                this.SetDesktopLocation(System.Windows.Forms.Cursor.Position.X - (this.Width / 2), System.Windows.Forms.Cursor.Position.Y - (this.Height / 2));

                if (this.Left + this.Width > Screen.PrimaryScreen.Bounds.Width)
                    this.Left = Screen.PrimaryScreen.Bounds.Width - this.Width;

                if (this.Top + this.Height > Screen.PrimaryScreen.Bounds.Height)
                    this.Top = Screen.PrimaryScreen.Bounds.Height - (this.Height);

                this.TopMost = true;
            }
        }

        private void TBBrightness_Scroll(object sender, EventArgs e)
        {
            if (!chkUseInternalSensor.Checked)
            {
                InternalSetBacklight((short)TBBrightness.Value);

                ModifyRegistry _registry = new ModifyRegistry();

                _registry.Write("Manual_Sensor_Setting", (object)TBBrightness.Value.ToString());
            }
        }

        private void BW_DoWork(object sender, DoWorkEventArgs e)
        {
            int _sensorReading = 0;
            int _value = 150;
            do
            {
                ssleep(20);
                if (chkUseInternalSensor.Checked && !_appExit)
                {
                    _sensorReading = (InternalReadLightSensor("L") + InternalReadLightSensor("R")) / 2;

                    SetTB(_sensorReading.ToString());
                    SetTB_X(read_motion_sensor(0).ToString());
                    SetTB_Y(read_motion_sensor(1).ToString());
                    SetTB_Z(read_motion_sensor(2).ToString());

                    //if (read_motion_sensor(0) < -20)
                    //{
                    //    System.Windows.Forms.SendKeys.SendWait("%{TAB}");
                    //}

                    if (_sensorReading >= 0 && _sensorReading < 15)
                    {
                        InternalSetBacklight((short)255);
                        SetS(255);
                    }

                    if (_sensorReading > 15 && _sensorReading < 25)
                    {
                        InternalSetBacklight((short)225);
                        SetS(225);
                    }

                    if (_sensorReading > 25 && _sensorReading < 50)
                    {
                        InternalSetBacklight((short)175);
                        SetS(175);
                    }

                    if (_sensorReading > 50 && _sensorReading < 100)
                    {
                        InternalSetBacklight((short)100);
                        SetS(100);
                    }

                    if (_sensorReading > 100 && _sensorReading < 150)
                    {
                        InternalSetBacklight((short)50);
                        SetS(50);
                    }

                    //if (_sensorReading > 150 && _sensorReading < 200)
                    //{
                    //    InternalSetBacklight((short)500);
                    //    SetS(50);
                    //}

                    if (_sensorReading > 150 && _sensorReading < 255)
                    {
                        InternalSetBacklight((short)0);
                        SetS(0);
                    }

                    try
                    {
                        _value = Convert.ToInt16(TBThreshold.Text);

                        if (_value >= 0 && _value <= 255)
                        {
                            if (_sensorReading >= _value)
                            {
                                InternalSetBacklight((short)0);
                                SetS(0);
                            }
                        }
                    }
                    catch { }

                }

                if (_appExit)
                    InternalSetBacklight(0);
            } while (!_appExit);
        }
        #endregion

        delegate void SetTBCB(string _text);
        delegate void SetTBCB_X(string _text);
        delegate void SetTBCB_Y(string _text);
        delegate void SetTBCB_Z(string _text);
        delegate void SetSCB(int _int);

        private void SetTB(string _TEXT)
        {
            if (this.InvokeRequired)
            {
                SetTBCB d = new SetTBCB(SetTB);
                this.Invoke(d, (object)_TEXT);
            }
            else
            {
                textBox1.Text = _TEXT;
            }
        }

        private void SetTB_X(string _TEXT)
        {
            if (this.InvokeRequired)
            {
                SetTBCB_X d = new SetTBCB_X(SetTB_X);
                this.Invoke(d, (object)_TEXT);
            }
            else
            {
                textBox_X.Text = _TEXT;
            }
        }

        private void SetTB_Y(string _TEXT)
        {
            if (this.InvokeRequired)
            {
                SetTBCB_Y d = new SetTBCB_Y(SetTB_Y);
                this.Invoke(d, (object)_TEXT);
            }
            else
            {
                textBox_Y.Text = _TEXT;
            }
        }

        private void SetTB_Z(string _TEXT)
        {
            if (this.InvokeRequired)
            {
                SetTBCB_Z d = new SetTBCB_Z(SetTB_Z);
                this.Invoke(d, (object)_TEXT);
            }
            else
            {
                textBox_Z.Text = _TEXT;
            }
        }

        private void SetS(int _INT)
        {
            if (this.InvokeRequired)
            {
                SetSCB d = new SetSCB(SetS);
                this.Invoke(d, (object)_INT);
            }
            else
            {
                TBBrightness.Value = _INT;
            }
        }

        #region Sensor & Backlight & Motion
        private int InternalSetBacklight(short _val)
        {
            short [] buffer = new short[2];
            short[] key = new short[4];
            key[0] = (short)76;
            key[1] = (short)75;
            key[2] = (short)83;
            key[3] = (short)66;
            
            buffer[0] = _val;
	        buffer[1] = 0x00;
            return writekey(key, 2, buffer);
        }

        private int InternalReadLightSensor(string _side) 
        {
	        short [] buffer = new short[6];
            short[] keyright = new short[4];
            keyright[0] = (short)65;
            keyright[1] = (short)76;
            keyright[2] = (short)86;
            if (_side == "L")
                keyright[3] = (short)48;
            if (_side == "R")
                keyright[3] = (short)49;

	        if (readkey(keyright, 6, buffer))
		        return buffer[2];
	        else
		        return -1;
        }

        private int init_motion_sensor() 
        {
	        short [] buffer = new short[2];
	        int retry = 50;
	        bool ret;

            short[] key = new short[4];
            key[0] = (short)77;
            key[1] = (short)79;
            key[2] = (short)67;
            key[3] = (short)78;
            
	        while (retry > 0) {
                ret = readkey(key, 2, buffer);
		        if (ret) {
			        if (buffer[0] > 0 || buffer[1] > 0) {
				        return 1;
			        }
		        }
		        else {
			        ssleep(1000);
			        retry--;
			        continue;
		        }
		        buffer[0] = 0xe0;
		        buffer[1] = 0;
                writekey(key, 2, buffer);
		        retry--;
		        ssleep(1000);
	        }
	        return 0;
        }

        private int read_motion_sensor(int index) 
        {
	        short [] buffer = new short[2];
	        bool ret = false;
            short[] key = new short[4];
            key[0] = (short)77;
            key[1] = (short)79;
            key[2] = (short)95;
            
	        switch (index) {
                case 0:
                key[3] = (short)88;
                ret = readkey(key, 2, buffer);
		        break;
            case 1:
                key[3] = (short)89;
                ret = readkey(key, 2, buffer);
		        break;
            case 2:
                key[3] = (short)90;
                ret = readkey(key, 2, buffer);
		        break;
	        }
	        //printf("%d %x %x\n", index, buffer[0], buffer[1]);
	        if (ret) {
		        return (short)(buffer[0] << 8 | buffer[1]);
	        }
	        else {
		        return 0x0FFFFFFF;
	        }
        }

        #endregion

        #region Port Access
        private void ssleep(int usec) 
        {
            System.Threading.Thread.Sleep(usec);
        }

        private int get_status() 
        {
            return inpout32.Inp32(0x304);
        }

        private bool waitfree(int num)
        {
            int c, pc = -1;
            int retry = 100;
            while ((c = get_status()) != num && (c = get_status()) != retry)
            {
                ssleep(10);
                retry--;
                if (pc != c)
                {
                    //printf("%x-%d:", c, retry);
                    pc = c;
                }
            }
            if (retry == 0)
            {
                //printf("Waitfree failed %x != %x.\n", c, num);
                return false;
            }
            /*else
                printf("Waitfree ok %x.\n", c);*/

            return true;
        }

        private int writekey(short[] key, short len, short[] buffer) 
        {
	        int i;

	        inpout32.Out32(0x304, 0x11);
	        //if (!waitfree(0x0c)) return 0;
            ssleep(20);
	        for (i = 0; i < 4; i++) {
		        inpout32.Out32(0x300, key[i]);
		        //if (!waitfree(0x04)) return 0;
                ssleep(20);
	        }

	        inpout32.Out32(0x300, len);

	        for (i = 0; i < len; i++) {
		        //if (!waitfree(0x04)) return 0;
                ssleep(20);
		        inpout32.Out32(0x300, buffer[i]);
	        }

	        return 1;
        }

        private bool readkey(short[] key, short len, short[] buffer) 
        {
	        int i; 
            short c;

            inpout32.Out32(0x304, 0x10);
	        //if (!waitfree(0x0c)) return false;
            ssleep(20);
	        for (i = 0; i < 4; i++) {
                inpout32.Out32(0x300, key[i]);
		        //if (!waitfree(0x04)) return false;
                ssleep(20);
	        }

            inpout32.Out32(0x300, len);

	        for (i = 0; i < len; i++) {
		        //if (!waitfree(0x05)) return false;
                ssleep(20);
                c = inpout32.Inp32(0x300);
		        buffer[i] = c;
	        }

	        return true;
        }
        #endregion

        private void chkUseInternalSensor_CheckedChanged(object sender, EventArgs e)
        {
            ModifyRegistry _registry = new ModifyRegistry();

            if (chkUseInternalSensor.Checked)
                _registry.Write("Use_Internal_Sensor", (object)"true");
            else
                _registry.Write("Use_Internal_Sensor", (object)"false");
        }

        private void TBThreshold_TextChanged(object sender, EventArgs e)
        {
            ModifyRegistry _registry = new ModifyRegistry();

            try
            {
                int _value = Convert.ToInt16(TBThreshold.Text);

                if (_value >= 0 && _value <= 255)
                {
                    _registry.Write("Off_Threshold", (object)TBThreshold.Text);
                }
                else
                    TBThreshold.Text = "";
            }
            catch 
            {
                TBThreshold.Text = "";
            }
        }
    }
}